from coopr.pyomo import *

model = ConcreteModel()

model.x = Var(within=Binary)
model.y = Var(within=Binary)
model.z = Var(within=Binary)

model.o = Objective(expr=model.x+2*model.y+3*model.z)

model.c1 = Constraint(expr=model.x+model.y >= 1)
model.c2 = Constraint(expr=model.x+model.z >= 1)
model.c3 = Constraint(expr=model.y+model.z >= 1)




from coopr.pyomo import *

model = ConcreteModel()

model.x = Var(bounds=(1,None), within=Integers)
model.y = Var(bounds=(1,None), within=Integers)

model.o = Objective(expr=model.x-model.x)

model.c = Constraint(expr=model.x+model.y >= 3)



# This is a computational proof that lp_unique1.py has a unique primal and dual solution.

from coopr.pyomo import *
from coopr.opt import SolverFactory

def get_model(flag, fixprim, fixdual):
    model = ConcreteModel()

    model.obj = Param(default=20)

    model.n = Param(default=7)
    model.m = Param(default=7)

    model.N = RangeSet(1,model.n)
    model.M = RangeSet(1,model.m)

    def c_rule(model, j):
        return 5 if j<5 else 9.0/2
    model.c = Param(model.N)

    def b_rule(model, i):
        if i == 4:
            i = 5
        elif i == 5:
            i = 4
        return 5 if i<5 else 7.0/2
    model.b = Param(model.M)

    def A_rule(model, i, j):
        if i == 4:
            i = 5
        elif i == 5:
            i = 4
        return 2 if i==j else 1
    model.A = Param(model.M, model.N)

    model.x = Var(model.N, within=NonNegativeReals)
    model.y = Var(model.M, within=NonNegativeReals)
    model.xx = Var([1,2], model.N, within=NonNegativeReals)
    model.yy = Var([1,2], model.M, within=NonNegativeReals)

    if flag:
        model.ydiff = Objective(expr=model.yy[2,fixdual] - model.yy[1,fixdual])
        
        def yext_rule(model, k):
            return sum(model.b[i]*model.yy[k,i] for i in model.M) == model.obj
        model.yext = Constraint([1,2])

        def dualcons_rule(model, k, j):
            return sum(model.A[i,j]*model.yy[k,i] for i in model.N) <= model.c[j]
        model.dualcons = Constraint([1,2], model.N)
    else:
        model.xdiff = Objective(expr=model.xx[2,fixprim] - model.xx[1,fixprim])

        def xext_rule(model, k):
            return sum(model.c[j]*model.xx[k,j] for j in model.N) == model.obj
        model.xext = Constraint([1,2])

        def primcons_rule(model, k, i):
            return sum(model.A[i,j]*model.xx[k,j] for j in model.M) >= model.b[i]
        model.primcons = Constraint([1,2], model.M)

    model.create()
    return model


#
# We iterate over all primal and dual indices, fixing each in turn.  Then, we solve a max-difference
# LP problem and verify that the difference is zero.  This demonstrates that there is no alternative 
# primal/dual solution, given that fixed value.
#
opt = SolverFactory('glpk')

for i in range(1,8):
    model = get_model(True, i, 1)
    results = opt.solve(model)
    if results.solution[0].objective['ydiff'].value != 0:
        print "ERROR: nonzero difference i=%d ydiff=%f" % (i, results.solution[0].objective['ydiff'].value)

for i in range(1,8):
    model = get_model(False, 1, i)
    results = opt.solve(model)
    if results.solution[0].objective['xdiff'].value != 0:
        print "ERROR: nonzero difference i=%d xdiff=%f" % (i, results.solution[0].objective['xdiff'].value)

print "SUCCESS!"



from coopr.pyomo import *

model = ConcreteModel()

model.d = Param(default=10)

def A_rule(model):
    return range(0,value(model.d))
model.A = Set()

def B_rule(model):
    return range(1,value(model.d)+1)
model.B = Set()

if 1>0:
    model.X1 = model.A*model.B
    model.X2 = model.A | model.B
    model.X3 = model.A ^ model.B
    model.X4 = model.B - model.A
    model.X5 = model.B & model.A

model.Y = Set(initialize=model.B - model.A)
model.Y.add('foo')

if 1>0:
    instance = model.create()
    print "X1", len(instance.X1)
    print instance.X1.data()
    print "X2", len(instance.X2)
    print instance.X2.data()
    print "X3", len(instance.X3)
    print instance.X3.data()
    print "X4", len(instance.X4)
    print instance.X4.data()
    print "X5", len(instance.X5)
    print instance.X5.data()

    print instance.Y.data()



from coopr.pyomo import *

model = ConcreteModel()

model.x = Var(bounds=(1,None))
model.y = Var(bounds=(1,None))

model.o = Objective(expr=-model.x-model.y, sense=maximize)

model.c = Constraint(expr=model.x+model.y <= 0)



import pyutilib.math
import sys
import coopr.opt


def mip_unique(results, data, options, name='unique-feasible'):
    for key in ['single solution', 'solution gap', 'solution status', 'objective name', 'objective name', 'objective value']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    #
    value = len(results.solution)
    data['single solution'] = value==1, '{0} solutions found for {1} MIP'.format(value, name)
    #
    value = results.solution[0].gap
    if value != coopr.opt.undefined:
        data['solution gap'] = pyutilib.math.approx_equal(value, 0.0, options.abstol, options.reltol), "'{0}' solution gap for {1} MIP solution".format(value, name)
    #
    value = results.solution[0].status
    data['solution status'] = value==coopr.opt.SolutionStatus.optimal, "'{0}' solution status for {1} MIP solution".format(value, name)
    #
    if len(results.solution[0].objective) > 0:
        objname = results.solution[0].objective.keys()[0]
        _objname = options._baseline.solution[0].objective.keys()[0]
        data['objective name'] = objname == _objname, "'{0}' objective name for {2} MIP solution (baseline={1})".format(objname, _objname, name)
        #
        value = results.solution[0].objective[objname].value
        data['objective value'] = pyutilib.math.approx_equal(value, options._baseline.solution[0].objective[_objname].value, options.abstol, options.reltol), "'{0}' objective value for {2} MIP solution (baseline={1})".format(value, options._baseline.solution[0].objective[_objname].value, name)


#
# Verify the problem name?
# Verify that we get reduced cost and duals (and others?)
# Verify problem stats?  (# constraints, # nonzeros, etc)
#
def mip_feasible_solution(results, data, options, name):
    for key in ['single objective', 'lower bound', 'upper bound', 'sense', 'solver status', 'termination condition', 'feasible solutions']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for {1} MIP'.format(value, name)
    #
    value = results.problem[0].lower_bound
    data['lower bound'] = pyutilib.math.approx_equal(value, options._baseline.problem[0].lower_bound, options.abstol, options.reltol), "'{0}' lower bound for {2} MIP (baseline={1})".format(value, options._baseline.problem[0].lower_bound, name)
    #
    value = results.problem[0].upper_bound
    data['upper bound'] = pyutilib.math.approx_equal(value, options._baseline.problem[0].upper_bound, options.abstol, options.reltol), "'{0}' upper bound for {2} MIP (baseline={1})".format(value, options._baseline.problem[0].upper_bound, name)
    #
    value = str(results.problem[0].sense)
    data['sense'] = value==options._baseline.problem[0].sense, "'{0}' sense for {2} MIP (baseline={1})".format(value, options._baseline.problem[0].sense, name)
    #
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for {1} MIP".format(value, name)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.optimal, "'{0}' termination condition for {1} MIP".format(value, name)
    #
    value = results.solver.error_rc
    data['error rc'] = value==0, "'{0}' error rc for {1} MIP".format(value, name)
    #
    #
    value = len(results.solution)
    data['feasible solutions'] = value>=1, '{0} solutions found for {1} MIP'.format(value, name)


def mip_feasible(results, data, options):
    mip_feasible_solution(results, data, options, 'feasible')

def mip_constant(results, data, options):
    mip_feasible_solution(results, data, options, 'constant')


def mip_unbounded(results, data, options):
    for key in ['single objective', 'no solution', 'solver status', 'termination condition']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for unbounded MIP'.format(value)
    #
    value = len(results.solution)
    data['no solution'] = value==0, '{0} solutions found for unbounded MIP'.format(value)
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for unbounded MIP".format(value)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.unbounded, "'{0}' termination condition for unbounded MIP".format(value)


def mip_infeasible(results, data, options):
    for key in ['single objective', 'solver status', 'termination condition']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for infeasible MIP'.format(value)
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for infeasible MIP".format(value)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.infeasible, "'{0}' termination condition for infeasible MIP".format(value)




#
# A LP model adapted from the Wikipedia description of the simplex algorithm:
#  http://en.wikipedia.org/wiki/Simplex_algorithm
#
from coopr.pyomo import *

model = ConcreteModel()

model.x = Var(within=NonNegativeReals)
model.y = Var(within=NonNegativeReals)
model.z = Var(within=NonNegativeReals)

model.Z = Objective(expr=-2*model.x -3*model.y -4*model.z)

model.c1 = Constraint(expr=3*model.x + 2*model.y +    model.z <= 10)
model.c2 = Constraint(expr=2*model.x + 5*model.y + 3* model.z <= 15)



#
# Test the Pyomo command-line interface
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep
datadir = os.path.normpath(currdir+'../../../../coopr.pyomo/examples/pyomo/p-median/')+os.sep
print datadir

from coopr.pyomo import *
import pyutilib.th as unittest
import coopr.pyomo.scripting.convert as convert
import re
import glob


@unittest.category('performance')
class Test(unittest.TestCase):

    def setUp(self):
        self.cwd = os.getcwd()
        os.chdir(currdir)

    def tearDown(self):
        os.chdir(self.cwd)


@unittest.category('performance')
class Test1(Test):

    def test1(self):
        res = convert.pyomo2nl(['--save-model',currdir+'test1.nl',datadir+'pmedian.py',datadir+'pmedian.dat'])
        if not os.path.exists(currdir+'test1.nl'):
            raise ValueError, "Missing file test1.nl generated in test1"
        os.remove(currdir+'test1.nl')
        if not res.options.max_memory is None:
            self.recordTestData('maximum memory used', res.options.max_memory)

    def test2(self):
        res = convert.pyomo2lp(['--save-model',currdir+'test2.lp',datadir+'pmedian.py',datadir+'pmedian.dat'])
        if not os.path.exists(currdir+'test2.lp'):
            raise ValueError, "Missing file test2.lp generated in test2"
        os.remove(currdir+'test2.lp')
        if not res.options.max_memory is None:
            self.recordTestData('maximum memory used', res.options.max_memory)

@unittest.category('performance')
class Test2(Test):
    pass


@unittest.category('performance')
class Test3(Test):
    pass


@unittest.nottest
def nl_test(self, name):
    fname = currdir+name+'.nl'
    root = name.split('_')[1]
    #print >>sys.stderr, fname
    res = convert.pyomo2nl(['--save-model',fname]+self.get_options(name)+[datadir+root+'.dat'])
    if not os.path.exists(fname):
        raise ValueError, "Missing file %s generated in test2" % fname
    os.remove(fname)
    if not res.options.max_memory is None:
        self.recordTestData('maximum memory used', res.options.max_memory)

@unittest.nottest
def lp_test(self, name):
    fname = currdir+name+'.lp'
    root = name.split('_')[1]
    #print >>sys.stderr, fname
    res = convert.pyomo2lp(['--save-model',fname]+self.get_options(name)+[datadir+root+'.dat'])
    if not os.path.exists(fname):
        raise ValueError, "Missing file %s generated in test2" % fname
    os.remove(fname)
    if not res.options.max_memory is None:
        self.recordTestData('maximum memory used', res.options.max_memory)


for i in range(8):
    name = 'test'+str(i+1)
    #
    Test2.add_fn_test(fn=nl_test, name='nl_pmedian.'+name, options=[datadir+'pmedian.py'])
    Test2.add_fn_test(fn=nl_test, name='nl-O_pmedian.'+name, options=['--disable-gc', datadir+'pmedian.py'] )
    if i < 4:
        Test3.add_fn_test(fn=nl_test, name='nl-memcov_pmedian.'+name, options=['--profile-memory','1',datadir+'pmedian.py'] )
    #
    Test2.add_fn_test(fn=lp_test, name='lp_pmedian.'+name, options=[datadir+'pmedian.py'])
    Test2.add_fn_test(fn=lp_test, name='lp-O_pmedian.'+name, options=['--disable-gc', datadir+'pmedian.py'] )
    if i < 4:
        Test3.add_fn_test(fn=lp_test, name='lp-memcov_pmedian.'+name, options=['--profile-memory','1',datadir+'pmedian.py'] )


if __name__ == "__main__":
    unittest.main()



# This problem has a unique primal and dual solution.

from coopr.pyomo import *

model = ConcreteModel()

model.n = Param(default=7)
model.m = Param(default=7)

model.N = RangeSet(1,model.n)
model.M = RangeSet(1,model.m)

def c_rule(model, j):
    return 5 if j<5 else 9.0/2
model.c = Param(model.N)

def b_rule(model, i):
    if i == 4:
        i = 5
    elif i == 5:
        i = 4
    return 5 if i<5 else 7.0/2
model.b = Param(model.M)

def A_rule(model, i, j):
    if i == 4:
        i = 5
    elif i == 5:
        i = 4
    return 2 if i==j else 1
model.A = Param(model.M, model.N)

model.x = Var(model.N, within=NonNegativeReals)
model.y = Var(model.M, within=NonNegativeReals)

model.cost = Objective(expr=summation(model.c, model.x))

def primalcon_rule(model, i):
    return sum(model.A[i,j]*model.x[j] for j in model.N) >= model.b[i]
model.primalcon = Constraint(model.M)






__all__ = ['test_solvers']

import re
import pyutilib.autotest
import pyutilib.component.core
import coopr.plugins.testdriver.pyomo
import os
import sys
import time
import csv
from os.path import abspath, dirname
import logging
import pprint
from pyutilib.misc import Bunch, Options
import pyutilib.th as unittest
currdir = dirname(abspath(__file__))+os.sep

import plugins

rootdir = None

def percentParts(numer, denom):
    if denom == 0:
        return "0"
    else:
        return int(round(100.0 * float(numer) / float(denom)))

def percentStrParts(numer, denom):
    return str(percentParts(numer, denom))+'%'


class SolverResultPrinter:

    def __init__(self, options):
        self.options = options

    def group_data(self, solverData, testResult):
        """
        Group the data bay the solver that produced it.

        @param solverData The 'data' object produced during solver testing;
                          contains information for solvers that passed their
                          unit tests.
        @param testResult The result object produced by the unittest framework.
        @return A dictionary mapping solver names to the elements in solverData.
        """

        ans = {}
        solverNameRe = re.compile("test_([a-zA-Z0-9_]*)_TEST")

        for test in testResult.runner.suite._tests:
            match = solverNameRe.match(test._testMethodName)
            if match is not None:
                solver = match.group(1)
                if solver not in ans:
                    ans[solver] = [x for x in solverData if x.name.startswith(solver+'_TEST')]

        return ans

    def write(self, solverData, testResult, filename):
        odir = os.getcwd()
        os.chdir(rootdir)
        with open(filename, 'w') as OUTPUT:
            print "\nWriting results summary to file '%s'" % abspath(filename)
            self.pprint(solverData, testResult, stream=OUTPUT)
            OUTPUT.close()
        os.chdir(odir)

    def pprint(self, solverData, testResult, stream = sys.stdout):
        """
        Print the results of solver testing in a human-readable format.

        This method is to be overridden by subclasses of SolverResultPrinter
        to provide different formats of printout, depending on the user's
        choices (given by command-line arguments).
        """
        raise Exception("Unimplemented")


class TabularResultPrinter(SolverResultPrinter):

    def pprint(self, solverData, testResult, stream = sys.stdout):
        groups = self.group_data(solverData, testResult)

        maxSolverNameLen = max([max(len(name) for name in groups), len("Solver")])
        fmtStr = "{{0:<{0}}}| {{1:>6}} | {{2:>10}} | {{3:>8}} | {{4:>8}}\n".format(maxSolverNameLen + 2)

        stream.write("\n")
        stream.write(fmtStr.format("Solver", "% Pass", "Sum % Pass", "# Pass", "# Run"))
        stream.write("=" * (maxSolverNameLen + 46) + "\n")

        allPass = 0
        allRun = 0
        for solver in sorted(groups.keys()):
            totalPass = 0
            totalRun = 0
            norm = {}
            for item in groups[solver]:
                if not item.problem in norm:
                    norm[item.problem] = []
                if item.result[0] is True:
                    norm[item.problem].append(1)
                    totalPass += 1
                else:
                    norm[item.problem].append(0)
                totalRun += 1
            if totalRun > 0:
                norm_val = 0.0
                for item in norm:
                    norm_val += sum(norm[item])/(1.0*len(norm[item]))
                stream.write(fmtStr.format(solver, percentStrParts(totalPass, totalRun), "%10.2f" % norm_val, totalPass, totalRun))
                allPass += totalPass
                allRun += totalRun
        stream.write("-" * (maxSolverNameLen + 46) + "\n")
        stream.write(fmtStr.format("Total", percentStrParts(allPass, allRun), "%10.2f" % len(norm), allPass, allRun) + "\n")


class VerboseResultPrinter(SolverResultPrinter):

    def pprint(self, solverData, testResult, stream = sys.stdout):
        groups = self.group_data(solverData, testResult)

        for solver in groups:
            stream.write("\n")
            stream.write("=== Solver: {0} ===\n".format(solver))
            if len(groups[solver]) == 0:
                stream.write('  No results\n')
            else:
                for item in groups[solver]:
                    if self.options.debug:
                        if item.result[0] is True:
                            result = 'PASS: '+item.result[1]
                        else:
                            result = 'FAIL: '+item.result[1]
                    elif item.result[0] is False:
                        result = 'FAIL: '+item.result[1]
                    else:
                        continue
                    stream.write('  - suite={0}\n    problem={1}\n    result="{2}"\n'.format(item.suite, item.problem, result))


class CSVResultPrinter(SolverResultPrinter):

    def pprint(self, solverData, testResult, stream = sys.stdout):
        groups = self.group_data(solverData, testResult)

        if not os.environ.get('HUDSON_URL',None) is None:
            colprefix = ['job','build','node']
            prefix = [os.environ['JOB_NAME'], os.environ['BUILD_NUMBER'], os.environ['NODE_NAME']]
        else:
            colprefix = []
            prefix=[]

        ans = [colprefix + ['classname', 'name', 'dataname', 'value']]
        allPass = 0
        allRun = 0
        for solver in groups:
            totalPass = 0
            totalRun = 0
            for item in groups[solver]:
                if item.result[0] is True:
                    totalPass += 1
                totalRun += 1
            if totalRun > 0:
                ans.append( prefix+['coopr.test-solvers', solver, '% pass', percentParts(totalPass, totalRun)] )
                allPass += totalPass
                allRun += totalRun
        ans.append( prefix+['coopr.test-solvers', 'AllSolvers', '% pass', percentParts(allPass, allRun)] )

        writer = csv.writer(stream)
        writer.writerows(ans)

class SolverTestRunner(unittest.TextTestRunner):

    class NullStream:

        def write(self, str):
            pass

        def flush(self):
            pass


    class SolverTestResult(unittest.TestResult):

        def __init__(self, runner):
            unittest.TestResult.__init__(self)
            self.runner = runner


    def __init__(self):
        unittest.TextTestRunner.__init__(self, verbosity=2)

    def run(self, test):
        # Prep for test run
        result = SolverTestRunner.SolverTestResult(self)
        testset = reduce(set.union, [set(t._tests) for t in test._tests], set())
        filteredSuite = unittest.TestSuite()
        if len(self.options.solver) == 0:
            prefix = re.compile("test_")
        else:
            prefix = re.compile("test_({0})_TEST".format("|".join(self.options.solver)))
        filteredSet = list( filter((lambda x : prefix.match(str(x._testMethodName)) is not None), testset) )
        filteredSet.sort()
        filteredSuite.addTests(filteredSet)

        logging.disable(logging.CRITICAL)
        # Temporarily disable printing
        if not self.options.debug:
            sys.stdout = SolverTestRunner.NullStream()
            sys.stderr = SolverTestRunner.NullStream()

        # Perform the run
        self.suite = filteredSuite
        filteredSuite(result)

        # Re-enable logging, printing
        if not self.options.debug:
            #logging.disable(logging.NOTSET)
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

        # Report results
        print "\nTest Driver Summary - {0} tests: {1} failures, {2} errors ({3} skipped)".format(result.testsRun, len(result.failures), len(result.errors), len(result.skipped))
        sys.stderr.flush()
        sys.stdout.flush()

        # Info about data
        TabularResultPrinter(self.options).pprint(SolverTestingDriver.data, result)
        if self.options.verbose:
            VerboseResultPrinter(self.options).pprint(SolverTestingDriver.data, result)
        if not self.options.csv is None:
            CSVResultPrinter(self.options).write(SolverTestingDriver.data, result, self.options.csv)
        return result


def test_solvers(options=None, argv=None):
    """
    The is the function executed by the command
        coopr test-solvers [solver ...]
    """
    global rootdir
    rootdir = os.getcwd()
    if argv is None:
        if options.debug:
            if len(options.solver) == 0:
                print "Testing all solvers"
            else:
                print "Testing solver", options.solver[0]
        # Over-ride the value of sys.argv, which is used by unittest.main()
        sys.argv=['test_solver']
    else:
        sys.argv=argv
    # Create the tests defined in the YAML configuration file
    autotest_options = Options()
    autotest_options.testname_format = "%s_TEST_%s"
    pyutilib.autotest.create_test_suites(filename=currdir+'test_solvers.yml', _globals=globals(), options=autotest_options)
    # Execute the tests, using a custom test runner
    runner = SolverTestRunner()
    runner.options = options
    unittest.main(module=globals()['__name__'], testRunner=runner)


class SolverTestingDriver(coopr.plugins.testdriver.pyomo.PyomoTestDriver):
    """
    A test driver plugin that is used to test solvers.
    """

    pyutilib.component.core.alias('coopr.test_solvers')

    data = []

    def run_test(self, testcase, name, options):
        sys.stdout.write('\n')
        sys.stdout.flush()
        # Setup option values that are specific to this test driver
        options.problemdir = options.currdir+'problems'+os.sep
        options.model = options.problemdir + options.problem+'.py'
        # Set the tester to work in the specific problem directory
        sys.path.append(options.problemdir)
        odir = os.getcwd()
        os.chdir(options.problemdir)
        # Add additional options to the pyomo commandline
        options.pyomo = "-v --stream-solver --solver-suffixes=all"
        # Call the run_test method in PyomoTestDriver
        try:
            coopr.plugins.testdriver.pyomo.PyomoTestDriver.run_test(self, testcase, name, options)
        except Exception, e:
            self.perform_tests(testcase, name, options)
            str = 'Pyomo Execution Error: '+str(e)
            print "ERROR:",str
            self.fail(str)
        os.chdir(odir)

    def recordData(self, testcase, name, options, data):
        """Record all data in the 'data' dictionary"""
        for key in data:
            self.data.append( Bunch(name=name, problem=options.problem, suite=options.suite, test=key, result=data[key]))

    def perform_tests(self, testcase, name, options):
        """
        Over-ride the tests that are performed by the Pyomo test driver.
        """
        import coopr.opt

        if options.tests is None:
            raise IOError, "No 'tests' option for test '%s'" % options.problem
        if not options.baseline is None:
            baseline = coopr.opt.SolverResults()
            try:
                baseline.read(filename=options.baseline)
            except Exception, e:
                s = "ERROR: Problem reading solver results in file '{0}': {1}".format(options.baseline, str(e))
                print s
                self.fail(s)
            options._baseline = baseline
        data = {}
        # For each test in options. test, call the corresponding plugin function
        try:
            for test in re.split('[ ]+',options.tests):
                fn = getattr(plugins, test)
                fn(options.results, data, options)
        except Exception, e:
            s = "ERROR: Problem performing solver checks in {2} for {0}: {1}".format(name, str(e), test)
            print s
            self.fail(s)
        # compute total check data
        if len(data) == 0:
            data['error'] = False, 'No checks performed'
        num = sum(1 for key in data if data[key][0] == False)
        # record total check data
        self.recordData(testcase, name, options, data)
        # Summarize all check failures
        if num > 0:
            print "Solver Check Failures:", name
            for key in data:
                if not data[key][0]:
                    print " ", data[key][1]
        else:
            print "No Solver Check Failures:", name




import pyutilib.math
import sys
import coopr.opt


def lp_bounds(results, data, options):
    name='unknown'
    for key in ['lower bound', 'upper bound', 'solution gap']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    #
    value = results.problem[0].lower_bound
    data['lower bound'] = pyutilib.math.approx_equal(value, options._baseline.problem[0].lower_bound, options.abstol, options.reltol), "'{0}' lower bound for {2} LP (baseline={1})".format(value, options._baseline.problem[0].lower_bound, name)
    #
    value = results.problem[0].upper_bound
    data['upper bound'] = pyutilib.math.approx_equal(value, options._baseline.problem[0].upper_bound, options.abstol, options.reltol), "'{0}' upper bound for {2} LP (baseline={1})".format(value, options._baseline.problem[0].upper_bound, name)
    #
    value = results.solution[0].gap
    if value != coopr.opt.undefined:
        data['solution gap'] = pyutilib.math.approx_equal(value, 0.0, options.abstol, options.reltol), "'{0}' solution gap for {1} LP solution".format(value, name)
    #

#
# Verify the problem name?
# Verify that we get reduced cost and duals (and others?)
# Verify problem stats?  (# constraints, # nonzeros, etc)
# Distinguish between feasible and optimal SolutionStatus values?
#
def lp_feasible_solution(results, data, options, name):
    for key in ['single objective', 'sense', 'solver status', 'termination condition', 'single solution', 'solution status', 'objective name', 'objective name', 'objective value']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for {1} LP'.format(value, name)
    #
    value = str(results.problem[0].sense)
    data['sense'] = value==options._baseline.problem[0].sense, "'{0}' sense for {2} LP (baseline={1})".format(value, options._baseline.problem[0].sense, name)
    #
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for {1} LP".format(value, name)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.optimal, "'{0}' termination condition for {1} LP".format(value, name)
    #
    value = results.solver.get('error_rc',None)
    data['error rc'] = value==0 or value==None, "'{0}' error rc for {1} LP".format(value, name)
    #
    #
    value = len(results.solution)
    data['single solution'] = value==1, '{0} solutions found for {1} LP'.format(value, name)
    #
    value = results.solution[0].status
    data['solution status'] = value==coopr.opt.SolutionStatus.optimal or value==coopr.opt.SolutionStatus.feasible, "'{0}' solution status for {1} LP solution".format(value, name)
    #
    if len(results.solution[0].objective) > 0:
        objname = results.solution[0].objective.keys()[0]
        _objname = options._baseline.solution[0].objective.keys()[0]
        data['objective name'] = objname == _objname, "'{0}' objective name for {2} LP solution (baseline={1})".format(objname, _objname, name)
        #
        value = results.solution[0].objective[objname].value
        data['objective value'] = pyutilib.math.approx_equal(value, options._baseline.solution[0].objective[_objname].value, options.abstol, options.reltol), "'{0}' objective value for {2} LP solution (baseline={1})".format(value, options._baseline.solution[0].objective[_objname].value, name)


def lp_feasible(results, data, options):
    lp_feasible_solution(results, data, options, 'feasible')

def lp_constant(results, data, options):
    lp_feasible_solution(results, data, options, 'constant')


def lp_unbounded(results, data, options):
    for key in ['single objective', 'no solution', 'solver status', 'termination condition']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for unbounded LP'.format(value)
    #
    value = len(results.solution)
    data['no solution'] = value==0, '{0} solutions found for unbounded LP'.format(value)
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for unbounded LP".format(value)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.unbounded, "'{0}' termination condition for unbounded LP".format(value)


def lp_infeasible(results, data, options):
    for key in ['single objective', 'solver status', 'termination condition']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    #
    value = results.problem[0].number_of_objectives
    data['single objective'] =  value==1, '{0} objectives found for infeasible LP'.format(value)
    #
    value = results.solver.status
    data['solver status'] = value==coopr.opt.SolverStatus.ok, "'{0}' solver status for infeasible LP".format(value)
    #
    value = results.solver.termination_condition
    data['termination condition'] = value==coopr.opt.TerminationCondition.infeasible, "'{0}' termination condition for infeasible LP".format(value)


def lp_unique_dual(results, data, options):
    for key in ['lower bound reduced cost suffix', 'upper bound reduced cost suffix', 'reduced cost suffix', 'dual suffix', 'slack suffix']:
        data[key] = False, 'Error: No results data available (%s)' % key
    if results is None:
        return
    if len(results.solution) == 0:
        return
    #
    soln = results.solution[0]
    for var in soln.variable:
        v = soln.variable[var]
        if 'Lrc' in v:
            data['lower bound reduced cost suffix'] =  True, '"lrc" suffix provided by solver'
        if 'Urc' in v:
            data['upper bound reduced cost suffix'] =  True, '"urc" suffix provided by solver'
        if 'Rc' in v:
            data['reduced cost suffix'] =  True, '"rc" suffix provided by solver'
    #
    for con in soln.constraint:
        c = soln.constraint[con]
        if 'Dual' in c:
            data['dual suffix'] =  True, '"dual" suffix provided by solver'
        if 'Slack' in c:
            data['slack suffix'] =  True, '"slack" suffix provided by solver'




from coopr.pyomo import *

model = ConcreteModel()

model.x = Var(within=Binary)
model.y = Var(within=Binary)
model.z = Var(within=Binary)

model.o = Objective(expr=-model.x-model.y-model.z)

model.c1 = Constraint(expr=model.x+model.y <= 1)
model.c2 = Constraint(expr=model.x+model.z <= 1)
model.c3 = Constraint(expr=model.y+model.z <= 1)

model.c4 = Constraint(expr=model.x+model.y+model.z >= 1.5)
